import { Resource, ResourceProps } from 'aws-cdk-lib';
import { FargateProfile, ICluster, KubernetesManifest } from 'aws-cdk-lib/aws-eks';
import { ILogGroup, RetentionDays } from 'aws-cdk-lib/aws-logs';
import { Construct } from 'constructs';
/**
 * Configuration for the FargateLogger resource.
 */
export interface FargateLoggerProps extends ResourceProps {
    /**
       * The EKS Cluster to configure Fargate logging for.
       */
    readonly cluster: ICluster;
    /**
       * A default list of Fargate profiles that should have permissions
       * configured. Alternatively profiles can be added at any time by calling
       * `addProfile`.
       */
    readonly fargateProfiles?: FargateProfile[];
    /**
       * The CloudWatch log group where Farget container logs will be sent.
       */
    readonly logGroup?: ILogGroup;
    /**
       * The prefix to add to the start of log streams created by the Fargate
       * logger.
       */
    readonly logStreamPrefix?: string;
    /**
       * The number of days logs sent to CloudWatch from Fluent Bit should be
       * retained before they are automatically removed.
       */
    readonly retention?: RetentionDays;
}
export declare class FargateLogger extends Resource {
    readonly cluster: ICluster;
    readonly logStreamPrefix: string;
    readonly retention: RetentionDays;
    readonly logGroup: ILogGroup;
    readonly resource: KubernetesManifest;
    constructor(scope: Construct, id: string, props: FargateLoggerProps);
    addFargateProfile(profile: FargateProfile): FargateLogger;
}
